@@ -4,7 +4,7 @@
  *	This procedure provides a version of the dlopen() for use
  *	in systems that don't support dynamic loading; It will
  *	work if the libraries libz.a, libpng.a, libjpeg.a,
- *	libtiff.a and libimg1.0.a are statically linked into
+ *	libtiff.a and libimg1.2.a are statically linked into
  *	a modified wish.
  *
  * Copyright (c) 1995-1996 Sun Microsystems, Inc.
@@ -28,34 +28,174 @@ extern int inflateInit_();
 extern int inflateReset();
 extern int inflateSync();
 extern int inflateEnd();
-extern int png_create_read_struct();
+extern int png_calculate_crc();
+extern int png_check_sig();
 extern int png_create_info_struct();
-extern int png_create_write_struct();
-extern int png_destroy_read_struct();
-extern int png_destroy_write_struct();
-extern int png_error();
-extern int png_get_channels();
-extern int png_get_error_ptr();
-extern int png_get_progressive_ptr();
-extern int png_get_rowbytes();
-extern int png_get_IHDR();
-extern int png_get_valid();
+extern int png_destroy_info_struct();
+extern int png_get_io_ptr();
+extern int png_info_destroy();
+extern int png_info_init();
 extern int png_init_io();
+extern int png_reset_crc();
+extern int png_set_sig_bytes();
+extern int png_sig_cmp();
+extern int png_zalloc();
+extern int png_zfree();
+extern int png_error();
+extern int png_get_error_ptr();
+extern int png_set_error_fn();
+extern int png_warning();
+extern int png_create_struct();
+extern int png_destroy_struct();
+extern int png_free();
+extern int png_malloc();
+extern int png_get_progressive_ptr();
+extern int png_process_IDAT_data();
+extern int png_process_data();
+extern int png_process_some_data();
+extern int png_progressive_combine_row();
+extern int png_push_crc_skip();
+extern int png_push_fill_buffer();
+extern int png_push_handle_tEXt();
+extern int png_push_handle_unknown();
+extern int png_push_handle_zTXt();
+extern int png_push_have_end();
+extern int png_push_have_info();
+extern int png_push_have_row();
+extern int png_push_process_row();
+extern int png_push_read_IDAT();
+extern int png_push_read_chunk();
+extern int png_push_read_sig();
+extern int png_push_read_tEXt();
+extern int png_push_read_zTXt();
+extern int png_push_restore_buffer();
+extern int png_push_save_buffer();
+extern int png_read_push_finish_row();
+extern int png_set_progressive_read_fn();
+extern int png_create_read_struct();
+extern int png_destroy_read_struct();
+extern int png_read_destroy();
+extern int png_read_end();
 extern int png_read_image();
 extern int png_read_info();
+extern int png_read_init();
+extern int png_read_row();
+extern int png_read_rows();
 extern int png_read_update_info();
-extern int png_set_interlace_handling();
+extern int png_start_read_image();
+extern int png_read_data();
 extern int png_set_read_fn();
-extern int png_set_text();
-extern int png_set_write_fn();
-extern int png_set_IHDR();
-extern int png_write_end();
-extern int png_write_info();
-extern int png_write_row();
+extern int png_build_gamma_table();
+extern int png_build_grayscale_palette();
+extern int png_do_background();
+extern int png_do_chop();
+extern int png_do_dither();
+extern int png_do_expand();
+extern int png_do_expand_palette();
+extern int png_do_gamma();
+extern int png_do_gray_to_rgb();
+extern int png_do_read_filler();
+extern int png_do_read_transformations();
+extern int png_do_unpack();
+extern int png_do_unshift();
+extern int png_init_read_transformations();
+extern int png_read_transform_info();
+extern int png_set_background();
+extern int png_set_dither();
 extern int png_set_expand();
-extern int png_set_filler();
+extern int png_set_gamma();
+extern int png_set_gray_to_rgb();
 extern int png_set_strip_16();
-
+extern int png_check_chunk_name();
+extern int png_combine_row();
+extern int png_crc_error();
+extern int png_crc_finish();
+extern int png_crc_read();
+extern int png_do_read_interlace();
+extern int png_get_uint_16();
+extern int png_get_uint_32();
+extern int png_handle_IEND();
+extern int png_handle_IHDR();
+extern int png_handle_PLTE();
+extern int png_handle_bKGD();
+extern int png_handle_cHRM();
+extern int png_handle_gAMA();
+extern int png_handle_hIST();
+extern int png_handle_oFFs();
+extern int png_handle_pHYs();
+extern int png_handle_sBIT();
+extern int png_handle_tEXt();
+extern int png_handle_tIME();
+extern int png_handle_tRNS();
+extern int png_handle_unknown();
+extern int png_handle_zTXt();
+extern int png_read_filter_row();
+extern int png_read_finish_row();
+extern int png_read_start_row();
+extern int png_set_crc_action();
+extern int png_do_bgr();
+extern int png_do_invert();
+extern int png_do_swap();
+extern int png_set_bgr();
+extern int png_set_filler();
+extern int png_set_interlace_handling();
+extern int png_set_invert_mono();
+extern int png_set_packing();
+extern int png_set_shift();
+extern int png_set_swap();
+extern int png_flush();
+extern int png_set_write_fn();
+extern int png_write_data();
+extern int png_convert_from_struct_tm();
+extern int png_convert_from_time_t();
+extern int png_create_write_struct();
+extern int png_destroy_write_struct();
+extern int png_set_compression_level();
+extern int png_set_compression_mem_level();
+extern int png_set_compression_method();
+extern int png_set_compression_strategy();
+extern int png_set_compression_window_bits();
+extern int png_set_filter();
+extern int png_set_flush();
+extern int png_write_destroy();
+extern int png_write_end();
+extern int png_write_flush();
+extern int png_write_image();
+extern int png_write_info();
+extern int png_write_init();
+extern int png_write_row();
+extern int png_write_rows();
+extern int png_do_pack();
+extern int png_do_shift();
+extern int png_do_write_transformations();
+extern int png_check_keyword();
+extern int png_do_write_interlace();
+extern int png_save_uint_16();
+extern int png_save_uint_32();
+extern int png_write_IDAT();
+extern int png_write_IEND();
+extern int png_write_IHDR();
+extern int png_write_PLTE();
+extern int png_write_bKGD();
+extern int png_write_cHRM();
+extern int png_write_chunk();
+extern int png_write_chunk_data();
+extern int png_write_chunk_end();
+extern int png_write_chunk_start();
+extern int png_write_filtered_row();
+extern int png_write_find_filter();
+extern int png_write_finish_row();
+extern int png_write_gAMA();
+extern int png_write_hIST();
+extern int png_write_oFFs();
+extern int png_write_pHYs();
+extern int png_write_sBIT();
+extern int png_write_sig();
+extern int png_write_start_row();
+extern int png_write_tEXt();
+extern int png_write_tIME();
+extern int png_write_tRNS();
+extern int png_write_zTXt();
 extern int jpeg_CreateCompress();
 extern int jpeg_abort_compress();
 extern int jpeg_destroy_compress();
@@ -247,6 +387,7 @@ extern int TIFFWriteRawTile();
 extern int TIFFWriteScanline();
 extern int TIFFWriteTile();
 
+
 static struct {
   char * name;
   int (*value)();
@@ -261,33 +402,174 @@ static struct {
   {"inflateReset", inflateReset},
   {"inflateSync", inflateSync},
   {"inflateEnd", inflateEnd},
-  {"png_create_read_struct", png_create_read_struct},
+  {"png_calculate_crc", png_calculate_crc},
+  {"png_check_sig", png_check_sig},
   {"png_create_info_struct", png_create_info_struct},
-  {"png_create_write_struct", png_create_write_struct},
-  {"png_destroy_read_struct", png_destroy_read_struct},
-  {"png_destroy_write_struct", png_destroy_write_struct},
-  {"png_error", png_error},
-  {"png_get_channels", png_get_channels},
-  {"png_get_error_ptr", png_get_error_ptr},
-  {"png_get_progressive_ptr", png_get_progressive_ptr},
-  {"png_get_rowbytes", png_get_rowbytes},
-  {"png_get_IHDR", png_get_IHDR},
-  {"png_get_valid", png_get_valid},
+  {"png_destroy_info_struct", png_destroy_info_struct},
+  {"png_get_io_ptr", png_get_io_ptr},
+  {"png_info_destroy", png_info_destroy},
+  {"png_info_init", png_info_init},
   {"png_init_io", png_init_io},
+  {"png_reset_crc", png_reset_crc},
+  {"png_set_sig_bytes", png_set_sig_bytes},
+  {"png_sig_cmp", png_sig_cmp},
+  {"png_zalloc", png_zalloc},
+  {"png_zfree", png_zfree},
+  {"png_error", png_error},
+  {"png_get_error_ptr", png_get_error_ptr},
+  {"png_set_error_fn", png_set_error_fn},
+  {"png_warning", png_warning},
+  {"png_create_struct", png_create_struct},
+  {"png_destroy_struct", png_destroy_struct},
+  {"png_free", png_free},
+  {"png_malloc", png_malloc},
+  {"png_get_progressive_ptr", png_get_progressive_ptr},
+  {"png_process_IDAT_data", png_process_IDAT_data},
+  {"png_process_data", png_process_data},
+  {"png_process_some_data", png_process_some_data},
+  {"png_progressive_combine_row", png_progressive_combine_row},
+  {"png_push_crc_skip", png_push_crc_skip},
+  {"png_push_fill_buffer", png_push_fill_buffer},
+  {"png_push_handle_tEXt", png_push_handle_tEXt},
+  {"png_push_handle_unknown", png_push_handle_unknown},
+  {"png_push_handle_zTXt", png_push_handle_zTXt},
+  {"png_push_have_end", png_push_have_end},
+  {"png_push_have_info", png_push_have_info},
+  {"png_push_have_row", png_push_have_row},
+  {"png_push_process_row", png_push_process_row},
+  {"png_push_read_IDAT", png_push_read_IDAT},
+  {"png_push_read_chunk", png_push_read_chunk},
+  {"png_push_read_sig", png_push_read_sig},
+  {"png_push_read_tEXt", png_push_read_tEXt},
+  {"png_push_read_zTXt", png_push_read_zTXt},
+  {"png_push_restore_buffer", png_push_restore_buffer},
+  {"png_push_save_buffer", png_push_save_buffer},
+  {"png_read_push_finish_row", png_read_push_finish_row},
+  {"png_set_progressive_read_fn", png_set_progressive_read_fn},
+  {"png_create_read_struct", png_create_read_struct},
+  {"png_destroy_read_struct", png_destroy_read_struct},
+  {"png_read_destroy", png_read_destroy},
+  {"png_read_end", png_read_end},
   {"png_read_image", png_read_image},
   {"png_read_info", png_read_info},
+  {"png_read_init", png_read_init},
+  {"png_read_row", png_read_row},
+  {"png_read_rows", png_read_rows},
   {"png_read_update_info", png_read_update_info},
-  {"png_set_interlace_handling", png_set_interlace_handling},
+  {"png_start_read_image", png_start_read_image},
+  {"png_read_data", png_read_data},
   {"png_set_read_fn", png_set_read_fn},
-  {"png_set_text", png_set_text},
-  {"png_set_write_fn", png_set_write_fn},
-  {"png_set_IHDR", png_set_IHDR},
-  {"png_write_end", png_write_end},
-  {"png_write_info", png_write_info},
-  {"png_write_row", png_write_row},
+  {"png_build_gamma_table", png_build_gamma_table},
+  {"png_build_grayscale_palette", png_build_grayscale_palette},
+  {"png_do_background", png_do_background},
+  {"png_do_chop", png_do_chop},
+  {"png_do_dither", png_do_dither},
+  {"png_do_expand", png_do_expand},
+  {"png_do_expand_palette", png_do_expand_palette},
+  {"png_do_gamma", png_do_gamma},
+  {"png_do_gray_to_rgb", png_do_gray_to_rgb},
+  {"png_do_read_filler", png_do_read_filler},
+  {"png_do_read_transformations", png_do_read_transformations},
+  {"png_do_unpack", png_do_unpack},
+  {"png_do_unshift", png_do_unshift},
+  {"png_init_read_transformations", png_init_read_transformations},
+  {"png_read_transform_info", png_read_transform_info},
+  {"png_set_background", png_set_background},
+  {"png_set_dither", png_set_dither},
   {"png_set_expand", png_set_expand},
-  {"png_set_filler", png_set_filler},
+  {"png_set_gamma", png_set_gamma},
+  {"png_set_gray_to_rgb", png_set_gray_to_rgb},
   {"png_set_strip_16", png_set_strip_16},
+  {"png_check_chunk_name", png_check_chunk_name},
+  {"png_combine_row", png_combine_row},
+  {"png_crc_error", png_crc_error},
+  {"png_crc_finish", png_crc_finish},
+  {"png_crc_read", png_crc_read},
+  {"png_do_read_interlace", png_do_read_interlace},
+  {"png_get_uint_16", png_get_uint_16},
+  {"png_get_uint_32", png_get_uint_32},
+  {"png_handle_IEND", png_handle_IEND},
+  {"png_handle_IHDR", png_handle_IHDR},
+  {"png_handle_PLTE", png_handle_PLTE},
+  {"png_handle_bKGD", png_handle_bKGD},
+  {"png_handle_cHRM", png_handle_cHRM},
+  {"png_handle_gAMA", png_handle_gAMA},
+  {"png_handle_hIST", png_handle_hIST},
+  {"png_handle_oFFs", png_handle_oFFs},
+  {"png_handle_pHYs", png_handle_pHYs},
+  {"png_handle_sBIT", png_handle_sBIT},
+  {"png_handle_tEXt", png_handle_tEXt},
+  {"png_handle_tIME", png_handle_tIME},
+  {"png_handle_tRNS", png_handle_tRNS},
+  {"png_handle_unknown", png_handle_unknown},
+  {"png_handle_zTXt", png_handle_zTXt},
+  {"png_read_filter_row", png_read_filter_row},
+  {"png_read_finish_row", png_read_finish_row},
+  {"png_read_start_row", png_read_start_row},
+  {"png_set_crc_action", png_set_crc_action},
+  {"png_do_bgr", png_do_bgr},
+  {"png_do_invert", png_do_invert},
+  {"png_do_swap", png_do_swap},
+  {"png_set_bgr", png_set_bgr},
+  {"png_set_filler", png_set_filler},
+  {"png_set_interlace_handling", png_set_interlace_handling},
+  {"png_set_invert_mono", png_set_invert_mono},
+  {"png_set_packing", png_set_packing},
+  {"png_set_shift", png_set_shift},
+  {"png_set_swap", png_set_swap},
+  {"png_flush", png_flush},
+  {"png_set_write_fn", png_set_write_fn},
+  {"png_write_data", png_write_data},
+  {"png_convert_from_struct_tm", png_convert_from_struct_tm},
+  {"png_convert_from_time_t", png_convert_from_time_t},
+  {"png_create_write_struct", png_create_write_struct},
+  {"png_destroy_write_struct", png_destroy_write_struct},
+  {"png_set_compression_level", png_set_compression_level},
+  {"png_set_compression_mem_level", png_set_compression_mem_level},
+  {"png_set_compression_method", png_set_compression_method},
+  {"png_set_compression_strategy", png_set_compression_strategy},
+  {"png_set_compression_window_bits", png_set_compression_window_bits},
+  {"png_set_filter", png_set_filter},
+  {"png_set_flush", png_set_flush},
+  {"png_write_destroy", png_write_destroy},
+  {"png_write_end", png_write_end},
+  {"png_write_flush", png_write_flush},
+  {"png_write_image", png_write_image},
+  {"png_write_info", png_write_info},
+  {"png_write_init", png_write_init},
+  {"png_write_row", png_write_row},
+  {"png_write_rows", png_write_rows},
+  {"png_do_pack", png_do_pack},
+  {"png_do_shift", png_do_shift},
+  {"png_do_write_transformations", png_do_write_transformations},
+  {"png_check_keyword", png_check_keyword},
+  {"png_do_write_interlace", png_do_write_interlace},
+  {"png_save_uint_16", png_save_uint_16},
+  {"png_save_uint_32", png_save_uint_32},
+  {"png_write_IDAT", png_write_IDAT},
+  {"png_write_IEND", png_write_IEND},
+  {"png_write_IHDR", png_write_IHDR},
+  {"png_write_PLTE", png_write_PLTE},
+  {"png_write_bKGD", png_write_bKGD},
+  {"png_write_cHRM", png_write_cHRM},
+  {"png_write_chunk", png_write_chunk},
+  {"png_write_chunk_data", png_write_chunk_data},
+  {"png_write_chunk_end", png_write_chunk_end},
+  {"png_write_chunk_start", png_write_chunk_start},
+  {"png_write_filtered_row", png_write_filtered_row},
+  {"png_write_find_filter", png_write_find_filter},
+  {"png_write_finish_row", png_write_finish_row},
+  {"png_write_gAMA", png_write_gAMA},
+  {"png_write_hIST", png_write_hIST},
+  {"png_write_oFFs", png_write_oFFs},
+  {"png_write_pHYs", png_write_pHYs},
+  {"png_write_sBIT", png_write_sBIT},
+  {"png_write_sig", png_write_sig},
+  {"png_write_start_row", png_write_start_row},
+  {"png_write_tEXt", png_write_tEXt},
+  {"png_write_tIME", png_write_tIME},
+  {"png_write_tRNS", png_write_tRNS},
+  {"png_write_zTXt", png_write_zTXt},
   {"jpeg_CreateCompress", jpeg_CreateCompress},
   {"jpeg_abort_compress", jpeg_abort_compress},
   {"jpeg_destroy_compress", jpeg_destroy_compress},
