@@ -3,7 +3,18 @@
 #ifndef _IMG
 #define _IMG
 
-#include "imgInt.h"
+#include "tcl.h"
+#include "tk.h"
+
+#define IMG_MAJOR_VERSION 1
+#define IMG_MINOR_VERSION 2
+#define IMG_RELEASE_LEVEL 1
+#define IMG_RELEASE_SERIAL 2
+
+#define IMG_VERSION "1.2"
+#define IMG_PATCH_LEVEL "1.2.2"
+
+#ifndef RESOURCE_INCLUDED
 
 #if defined(__WIN32__)
 #   define WIN32_LEAN_AND_MEAN
@@ -24,6 +35,7 @@
 
 EXTERN EXPORT(int,Img_Init) _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN EXPORT(int,Img_SafeInit) _ANSI_ARGS_((Tcl_Interp *interp));
-EXTERN EXPORT(int,Img_InitStandAlone) _ANSI_ARGS_((Tcl_Interp *interp));
 
-#endif _IMG
+#endif /* RESOURCE_INCLUDED */
+
+#endif /* _IMG */
