@@ -18,19 +18,19 @@
  */
 
 static int ChnMatchBMP _ANSI_ARGS_((Tcl_Channel chan, char *fileName,
-	struct Tcl_Obj *format, int *widthPtr, int *heightPtr));
-static int ObjMatchBMP _ANSI_ARGS_((struct Tcl_Obj *dataObj,
-	struct Tcl_Obj *format, int *widthPtr, int *heightPtr));
+	Tcl_Obj *format, int *widthPtr, int *heightPtr));
+static int ObjMatchBMP _ANSI_ARGS_((Tcl_Obj *dataObj,
+	Tcl_Obj *format, int *widthPtr, int *heightPtr));
 static int ChnReadBMP _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Channel chan,
-	char *fileName, struct Tcl_Obj *format, Tk_PhotoHandle imageHandle,
+	char *fileName, Tcl_Obj *format, Tk_PhotoHandle imageHandle,
 	int destX, int destY, int width, int height, int srcX, int srcY));
-static int ObjReadBMP _ANSI_ARGS_((Tcl_Interp *interp, struct Tcl_Obj *dataObj,
-	struct Tcl_Obj *format, Tk_PhotoHandle imageHandle,
+static int ObjReadBMP _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj *dataObj,
+	Tcl_Obj *format, Tk_PhotoHandle imageHandle,
 	int destX, int destY, int width, int height, int srcX, int srcY));
 static int ChnWriteBMP _ANSI_ARGS_((Tcl_Interp *interp, char *filename,
-	struct Tcl_Obj *format, Tk_PhotoImageBlock *blockPtr));
+	Tcl_Obj *format, Tk_PhotoImageBlock *blockPtr));
 static int StringWriteBMP _ANSI_ARGS_((Tcl_Interp *interp,
-	Tcl_DString *data, struct Tcl_Obj *format,
+	Tcl_DString *data, Tcl_Obj *format,
 	Tk_PhotoImageBlock *blockPtr));
 
 Tk_PhotoImageFormat imgFmtBMP = {
@@ -60,7 +60,7 @@ static void putint _ANSI_ARGS_((MFile *handle, int i));
 static int ChnMatchBMP(chan, fileName, format, widthPtr, heightPtr)
     Tcl_Channel chan;
     char *fileName;
-    struct Tcl_Obj *format;
+    Tcl_Obj *format;
     int *widthPtr, *heightPtr;
 {
     MFile handle;
@@ -72,8 +72,8 @@ static int ChnMatchBMP(chan, fileName, format, widthPtr, heightPtr)
 }
 
 static int ObjMatchBMP(dataObj, format, widthPtr, heightPtr)
-    struct Tcl_Obj *dataObj;
-    struct Tcl_Obj *format;
+    Tcl_Obj *dataObj;
+    Tcl_Obj *format;
     int *widthPtr, *heightPtr;
 {
     MFile handle;
@@ -159,7 +159,7 @@ static int ChnReadBMP(interp, chan, fileName, format, imageHandle,
     Tcl_Interp *interp;
     Tcl_Channel chan;
     char *fileName;
-    struct Tcl_Obj *format;
+    Tcl_Obj *format;
     Tk_PhotoHandle imageHandle;
     int destX, destY;
     int width, height;
@@ -177,8 +177,8 @@ static int ChnReadBMP(interp, chan, fileName, format, imageHandle,
 static int ObjReadBMP(interp, data, format, imageHandle,
 	destX, destY, width, height, srcX, srcY)
     Tcl_Interp *interp;
-    struct Tcl_Obj *data;
-    struct Tcl_Obj *format;
+    Tcl_Obj *data;
+    Tcl_Obj *format;
     Tk_PhotoHandle imageHandle;
     int destX, destY;
     int width, height;
@@ -242,6 +242,7 @@ static int CommonReadBMP(interp, handle, imageHandle, destX, destY,
     block.offset[0] = 2;
     block.offset[1] = 1;
     block.offset[2] = 0;
+    block.offset[3] = block.offset[0];
     switch (numBits) {
 	case 24:
 	    block.pixelPtr = line + srcX*3;
@@ -332,20 +333,17 @@ error:
 static int ChnWriteBMP(interp, filename, format, blockPtr)
     Tcl_Interp *interp;
     char *filename;
-    struct Tcl_Obj *format;
+    Tcl_Obj *format;
     Tk_PhotoImageBlock *blockPtr;
 {
     Tcl_Channel chan;
     MFile handle;
     int result;
 
-    chan = Tcl_OpenFileChannel(interp, filename, "w", 0644);
+    chan = ImgOpenFileChannel(interp, filename, 0644);
     if (!chan) {
 	return TCL_ERROR;
     }
-    if (Tcl_SetChannelOption(interp, chan, "-translation", "binary") != TCL_OK) {
-	return TCL_ERROR;
-    }
 
     handle.data = (char *) chan;
     handle.state = IMG_CHAN;
@@ -360,7 +358,7 @@ static int ChnWriteBMP(interp, filename, format, blockPtr)
 static int StringWriteBMP(interp, data, format, blockPtr)
     Tcl_Interp *interp;
     Tcl_DString *data;
-    struct Tcl_Obj *format;
+    Tcl_Obj *format;
     Tk_PhotoImageBlock *blockPtr;
 {
     MFile handle;
@@ -392,14 +390,47 @@ static int CommonWriteBMP(interp, handle, blockPtr)
     int bperline, nbytes, ncolors, i, x, y, greenOffset, blueOffset, alphaOffset;
     unsigned char *imagePtr, *pixelPtr;
     unsigned char buf[4];
+    int colors[256];
+    int testnum = 10;
 
-    if (blockPtr->offset[0] == blockPtr->offset[1]) {
-	/* we have a grayscale image */
-	nbytes = 1;
-	ncolors = 256;
+    greenOffset = blockPtr->offset[1] - blockPtr->offset[0];
+    blueOffset = blockPtr->offset[2] - blockPtr->offset[0];
+    alphaOffset = blockPtr->offset[0];
+    if (alphaOffset < blockPtr->offset[2]) {
+	alphaOffset = blockPtr->offset[2];
+    }
+    if (++alphaOffset < blockPtr->pixelSize) {
+	alphaOffset -= blockPtr->offset[0];
     } else {
-	nbytes = 3;
-	ncolors = 0;
+	alphaOffset = 0;
+    }
+    ncolors = 0;
+    if (greenOffset || blueOffset) {
+	for (y = 0; ncolors <= 256 && y < blockPtr->height; y++) {
+	    pixelPtr = blockPtr->pixelPtr + y*blockPtr->pitch + blockPtr->offset[0];
+	    for (x=0; ncolors <= 256 && x<blockPtr->width; x++) {
+		int pixel;
+		if (alphaOffset && (pixelPtr[alphaOffset] == 0))
+		    pixel = 0xd9d9d9;
+		else
+		    pixel = (pixelPtr[0]<<16) | (pixelPtr[greenOffset]<<8) | pixelPtr[blueOffset];
+		for (i = 0; i < ncolors && pixel != colors[i]; i++);
+		if ((i == ncolors) && (ncolors < 256))
+		    colors[ncolors++] = pixel;
+		pixelPtr += blockPtr->pixelSize;
+	    }
+	}
+	if (ncolors <= 256 && (blockPtr->width * blockPtr->height >= 512)) {
+            while (ncolors < 256) {
+		colors[ncolors++] = 0;
+	    }
+	    nbytes = 1;
+	} else {
+	    nbytes = 3;
+	    ncolors = 0;
+	}
+    } else {
+        nbytes = 1;
     }
 
     bperline = ((blockPtr->width  * nbytes + 3) / 4) * 4;
@@ -420,7 +451,7 @@ static int CommonWriteBMP(interp, handle, blockPtr)
     putint(handle, ncolors);
 
     for (i = 0; i < ncolors ; i++) {
-	putint(handle, i*65793);
+	putint(handle, colors[i]);
     }
 
     bperline -= blockPtr->width * nbytes;
@@ -441,7 +472,15 @@ static int CommonWriteBMP(interp, handle, blockPtr)
     for (y = 0; y < blockPtr->height; y++) {
 	pixelPtr = imagePtr -= blockPtr->pitch;
 	for (x=0; x<blockPtr->width; x++) {
-	    if (alphaOffset && (pixelPtr[alphaOffset] == 0)) {
+	    if (ncolors) {
+		int pixel;
+		if (alphaOffset && (pixelPtr[alphaOffset] == 0))
+		    pixel = 0xd9d9d9;
+		else
+		    pixel = (pixelPtr[0]<<16)|(pixelPtr[greenOffset]<<8)|pixelPtr[blueOffset];
+		for (i = 0; i < ncolors && pixel != colors[i]; i += 1);
+		buf[0] = i;
+	    } else if (alphaOffset && (pixelPtr[alphaOffset] == 0)) {
 		buf[0] = buf[1] = buf[2] = 0xd9;
 	    } else {
 		buf[0] = pixelPtr[blueOffset];
@@ -449,6 +488,9 @@ static int CommonWriteBMP(interp, handle, blockPtr)
 		buf[2] = pixelPtr[0];
 	    }
 	    ImgWrite(handle, (char *) buf, nbytes);
+	    if (testnum >0) {
+		testnum--;
+	    }
 	    pixelPtr += blockPtr->pixelSize;
 	}
 	if (bperline) {
