@@ -2,10 +2,21 @@
  *	imgInit.tcl
  */
 
-#include "img.h"
 #include <string.h>
 #include <stdlib.h>
 
+#include "imgInt.h"
+
+#ifndef USE_TCL_STUBS
+#undef Tcl_InitStubs
+#define Tcl_InitStubs(a,b,c) Tcl_PkgRequire(a,"Tcl",TCL_VERSION,1)
+#endif
+
+#ifndef USE_TK_STUBS
+#undef Tk_InitStubs
+#define Tk_InitStubs(a,b,c) Tcl_PkgRequire(a,"Tk",TK_VERSION,1)
+#endif
+
 /*
  * Declarations for functions defined in this file.
  */
@@ -14,9 +25,9 @@ static int char64 _ANSI_ARGS_((int c));
 
 #ifdef ALLOW_B64
 static int tob64 _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
-		int argc, struct Tcl_Obj *CONST objv[]));
+		int argc, Tcl_Obj *CONST objv[]));
 static int fromb64 _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
-		int argc, struct Tcl_Obj *CONST objv[]));
+		int argc, Tcl_Obj *CONST objv[]));
 #endif
 
 extern Tk_ImageType		imgPixmapImageType;
@@ -26,12 +37,14 @@ extern Tk_PhotoImageFormat	imgFmtGIF;
 extern Tk_PhotoImageFormat	imgFmtJPEG;
 extern Tk_PhotoImageFormat	imgFmtPNG;
 extern Tk_PhotoImageFormat	imgFmtPS;
+extern Tk_PhotoImageFormat	imgFmtPDF;
 extern Tk_PhotoImageFormat	imgFmtRAS;
 extern Tk_PhotoImageFormat	imgFmtRAW;
 extern Tk_PhotoImageFormat	imgFmtRGB;
 extern Tk_PhotoImageFormat	imgFmtTIFF;
 extern Tk_PhotoImageFormat	imgFmtXBM;
 extern Tk_PhotoImageFormat	imgFmtXPM;
+extern Tk_PhotoImageFormat	imgFmtWin;
 
 static Tk_PhotoImageFormat *Formats[] = {
 	&imgFmtTIFF,
@@ -39,22 +52,23 @@ static Tk_PhotoImageFormat *Formats[] = {
 /*	&imgFmtRAS,*/
 /*	&imgFmtRGB,*/
 	&imgFmtPS,
+	&imgFmtPDF,
 	&imgFmtXBM,
 	&imgFmtXPM,
 	&imgFmtBMP,
 	&imgFmtJPEG,
 	&imgFmtPNG,
 	&imgFmtGIF,
+	&imgFmtWin,
 	(Tk_PhotoImageFormat *) NULL};
 
 /*
  * The variable "initialized" contains flags indicating which
  * version of Tcl or Perl we are running:
  *
- *      IMG_PERL	perl
  *	IMG_TCL		Tcl
- *	IMG_CHAN	using Tcl_Channel's in stead of files
- *	IMG_OBJS	using Tcl_Obj's in stead of char *
+ *	IMG_OBJS	using Tcl_Obj's in stead of char* (patch.tk8)
+ *      IMG_PERL	perl
  *
  * These flags will be determined at runtime (except the IMG_PERL
  * flag, for now), so we can use the same dynamic library for all
@@ -84,12 +98,15 @@ EXPORT(int,Img_Init)(interp)
     Tk_PhotoImageFormat **formatPtr = Formats;
     char *version;
 
-    if ((version = Tcl_PkgRequire(interp, "Tk", (char *) NULL, 0)) == NULL) {
+    if ((version = Tcl_InitStubs(interp, "8.0", 0)) == NULL) {
+	return TCL_ERROR;
+    }
+    if (Tk_InitStubs(interp, "8.0", 0) == NULL) {
 	return TCL_ERROR;
     }
 
     if (!initialized) {
-	if (!(initialized = ImgObjInit(interp, version))) {
+	if (!(initialized = ImgObjInit(interp))) {
 	    return TCL_ERROR;
 	}
 	while(*formatPtr) {
@@ -103,7 +120,7 @@ EXPORT(int,Img_Init)(interp)
     Tcl_CreateObjCommand(interp,"img_to_base64", tob64, (ClientData) NULL, NULL);
     Tcl_CreateObjCommand(interp,"img_from_base64", fromb64, (ClientData) NULL, NULL);
 #endif
-    return Tcl_PkgProvide(interp,"Img","1.1");
+    return Tcl_PkgProvide(interp,"Img", IMG_PATCH_LEVEL);
 }
 
 EXPORT(int,Img_SafeInit)(interp)
@@ -112,12 +129,6 @@ EXPORT(int,Img_SafeInit)(interp)
     return Img_Init(interp);
 }
 
-EXPORT(int,Img_InitStandAlone)(interp)
-    Tcl_Interp *interp;
-{
-    return Img_Init(interp);
-}
-
 /*
  *--------------------------------------------------------------------------
  * char64 --
@@ -187,11 +198,10 @@ char64(c)
 int
 ImgRead(handle, dst, count)
     MFile *handle;	/* mmdecode "file" handle */
-    VOID *dst;		/* where to put the result */
+    char *dst;		/* where to put the result */
     int count;		/* number of bytes */
 {
     register int i, c;
-    char *d;
     switch (handle->state) {
       case IMG_STRING:
 	if (count > handle->length) {
@@ -204,15 +214,11 @@ ImgRead(handle, dst, count)
 	}
 	return count;
       case IMG_CHAN:
-	if (initialized & IMG_CHAN)
-	    return Tcl_Read((Tcl_Channel) handle->data, dst, count);
-	else
-	    return fread(dst, 1, count, (FILE *) handle->data);
+	return Tcl_Read((Tcl_Channel) handle->data, dst, count);
     }
 
-    d = (char *) dst;
     for(i=0; i<count && (c=ImgGetc(handle)) != IMG_DONE; i++) {
-	*d++ = c;
+	*dst++ = c;
     }
     return i;
 }
@@ -439,13 +445,7 @@ ImgSeek(handle, off, whence)
     int off;
     int whence;
 {
-    if (initialized & IMG_CHAN) {
-	return Tcl_Seek((Tcl_Channel) handle->data, off, whence);
-    } else if (fseek((FILE *) handle->data, (long) off, whence)) {
-	return -1;
-    } else {
-	return (int) ftell((FILE *) handle->data);
-    }
+    return Tcl_Seek((Tcl_Channel) handle->data, off, whence);
 }
 
 /*
@@ -491,7 +491,7 @@ ImgWriteInit(buffer, handle)
 
 int
 ImgReadInit(data, c, handle)
-    struct Tcl_Obj *data;	/* string containing initial mmencoded data */
+    Tcl_Obj *data;		/* string containing initial mmencoded data */
     int c;
     MFile *handle;		/* mmdecode "file" handle */
 {
@@ -534,7 +534,7 @@ int tob64(clientData, interp, argc, objv)
     ClientData clientData;
     Tcl_Interp *interp;
     int argc;
-    struct Tcl_Obj *CONST objv[];
+    Tcl_Obj *CONST objv[];
 {
     Tcl_DString dstring;
     MFile handle;
@@ -543,18 +543,14 @@ int tob64(clientData, interp, argc, objv)
     int len;
 
     if (argc != 2) {
-	Tcl_AppendResult(interp, "wrong num of args: should be \"",
-		Tcl_GetStringFromObj(objv[0], &len)," filename\"", (char *) NULL);
+	Tcl_WrongNumArgs(interp, 1, objv, "filename");
 	return TCL_ERROR;
     }
 
-    chan = Tcl_OpenFileChannel(interp, Tcl_GetStringFromObj(objv[1], &len), "r", 0);
+    chan = ImgOpenFileChannel(interp, Tcl_GetStringFromObj(objv[1], &len), 0);
     if (!chan) {
 	return TCL_ERROR;
     }
-    if (Tcl_SetChannelOption(interp, chan, "-translation", "binary") != TCL_OK) {
-	return TCL_ERROR;
-    }
 
     Tcl_DStringInit(&dstring);
     ImgWriteInit(&dstring, &handle);
@@ -596,7 +592,7 @@ int fromb64(clientData, interp, argc, objv)
     ClientData clientData;
     Tcl_Interp *interp;
     int argc;
-    struct Tcl_Obj *CONST objv[];
+    Tcl_Obj *CONST objv[];
 {
     MFile handle;
     Tcl_Channel chan;
@@ -604,19 +600,14 @@ int fromb64(clientData, interp, argc, objv)
     int len;
 
     if (argc != 3) {
-	Tcl_AppendResult(interp, "wrong num of args: should be \"",
-		Tcl_GetStringFromObj(objv[0], &len)," filename data\"",
-		(char *) NULL);
+	Tcl_WrongNumArgs(interp, 1, objv, "filename data");
 	return TCL_ERROR;
     }
 
-    chan = Tcl_OpenFileChannel(interp, Tcl_GetStringFromObj(objv[1], &len), "w", 0644);
+    chan = ImgOpenFileChannel(interp, Tcl_GetStringFromObj(objv[1], &len), 0644);
     if (!chan) {
 	return TCL_ERROR;
     }
-    if (Tcl_SetChannelOption(interp, chan, "-translation", "binary") != TCL_OK) {
-	return TCL_ERROR;
-    }
 
     handle.data = Tcl_GetStringFromObj(objv[2], &handle.length);
     handle.state = 0;
