@@ -74,7 +74,7 @@
 #endif
 
 #ifdef __WIN32__
-#define JPEG_LIB_NAME "jpeg.dll"
+#define JPEG_LIB_NAME "jpeg62.dll"
 #endif
 
 #ifndef JPEG_LIB_NAME
@@ -86,22 +86,22 @@
  */
 
 static int	ChnMatchJPEG _ANSI_ARGS_((Tcl_Channel chan, char *fileName,
-		    struct Tcl_Obj *format, int *widthPtr, int *heightPtr));
-static int	ObjMatchJPEG _ANSI_ARGS_((struct Tcl_Obj *dataObj,
-		    struct Tcl_Obj *format, int *widthPtr, int *heightPtr));
+		    Tcl_Obj *format, int *widthPtr, int *heightPtr));
+static int	ObjMatchJPEG _ANSI_ARGS_((Tcl_Obj *dataObj,
+		    Tcl_Obj *format, int *widthPtr, int *heightPtr));
 static int	ChnReadJPEG _ANSI_ARGS_((Tcl_Interp *interp,
-		    Tcl_Channel chan, char *fileName, struct Tcl_Obj *format,
+		    Tcl_Channel chan, char *fileName, Tcl_Obj *format,
 		    Tk_PhotoHandle imageHandle, int destX, int destY,
 		    int width, int height, int srcX, int srcY));
 static int	ObjReadJPEG _ANSI_ARGS_((Tcl_Interp *interp,
-		    struct Tcl_Obj *dataObj, struct Tcl_Obj *format,
+		    Tcl_Obj *dataObj, Tcl_Obj *format,
 		    Tk_PhotoHandle imageHandle, int destX, int destY,
 		    int width, int height, int srcX, int srcY));
 static int	ChnWriteJPEG _ANSI_ARGS_((Tcl_Interp *interp,
-		    char *fileName, struct Tcl_Obj *format,
+		    char *fileName, Tcl_Obj *format,
 		    Tk_PhotoImageBlock *blockPtr));
 static int	StringWriteJPEG _ANSI_ARGS_((Tcl_Interp *interp,
-		    Tcl_DString *dataPtr, struct Tcl_Obj *format,
+		    Tcl_DString *dataPtr, Tcl_Obj *format,
 		    Tk_PhotoImageBlock *blockPtr));
 
 Tk_PhotoImageFormat imgFmtJPEG = {
@@ -151,13 +151,13 @@ struct my_error_mgr {		/* Extended libjpeg error manager */
 static int	CommonMatchJPEG _ANSI_ARGS_((MFile *handle,
 		    int *widthPtr, int *heightPtr));
 static int	CommonReadJPEG _ANSI_ARGS_((Tcl_Interp *interp,
-		    j_decompress_ptr cinfo, struct Tcl_Obj *format,
+		    j_decompress_ptr cinfo, Tcl_Obj *format,
 		    Tk_PhotoHandle imageHandle, int destX, int destY,
 		    int width, int height, int srcX, int srcY));
 static int	CommonWriteJPEG _ANSI_ARGS_((Tcl_Interp *interp,
-		    j_compress_ptr cinfo, struct Tcl_Obj *format,
+		    j_compress_ptr cinfo, Tcl_Obj *format,
 		    Tk_PhotoImageBlock *blockPtr));
-static void	jpeg_obj_src _ANSI_ARGS_((j_decompress_ptr, struct Tcl_Obj *));
+static void	jpeg_obj_src _ANSI_ARGS_((j_decompress_ptr, Tcl_Obj *));
 static void	jpeg_channel_src _ANSI_ARGS_((j_decompress_ptr, Tcl_Channel));
 static boolean	fill_input_buffer _ANSI_ARGS_((j_decompress_ptr));
 static void	skip_input_data _ANSI_ARGS_((j_decompress_ptr, long));
@@ -189,7 +189,7 @@ static struct JpegFunctions {
     int (* read_header) _ANSI_ARGS_((j_decompress_ptr, int));
     JDIMENSION (* read_scanlines) _ANSI_ARGS_((j_decompress_ptr,
 			JSAMPARRAY, JDIMENSION));
-    int (* resync_to_restart) _ANSI_ARGS_((j_decompress_ptr, int));
+    boolean (* resync_to_restart) _ANSI_ARGS_((j_decompress_ptr, int));
     int (* set_defaults) _ANSI_ARGS_((j_compress_ptr));
     int (* start_compress) _ANSI_ARGS_((j_compress_ptr, int));
     int (* start_decompress) _ANSI_ARGS_((j_decompress_ptr));
@@ -332,7 +332,7 @@ load_jpeg_library(interp)
 load_failed:
       if (interp) {
 	Tcl_AppendResult(interp, "couldn't load \"", JPEG_LIB_NAME,
-		"\": please upgrade to at least version 6b", (char *) NULL);
+		"\": please upgrade to at least version 6a", (char *) NULL);
       }
       ImgLoadFailed(&jpeg.handle);
       return TCL_ERROR;
@@ -394,7 +394,7 @@ ImgLoadJpegLibrary()
     return TCL_ERROR;
 }
 
-int Imgjpeg_resync_to_restart _ANSI_ARGS_((j_decompress_ptr, int));
+boolean Imgjpeg_resync_to_restart _ANSI_ARGS_((j_decompress_ptr, int));
 JDIMENSION Imgjpeg_read_scanlines _ANSI_ARGS_((j_decompress_ptr,
 			JSAMPARRAY, JDIMENSION));
 int Imgjpeg_set_colorspace _ANSI_ARGS_((j_compress_ptr, J_COLOR_SPACE));
@@ -455,7 +455,7 @@ Imgjpeg_CreateDecompress(cinfo, version, size)
     return jpeg.CreateDecompress(cinfo, version, size);
 }
 
-int
+boolean
 Imgjpeg_resync_to_restart(a,b)
     j_decompress_ptr a;
     int b;
@@ -619,7 +619,7 @@ static int
 ChnMatchJPEG(chan, fileName, format, widthPtr, heightPtr)
     Tcl_Channel chan;		/* The image channel, open for reading. */
     char *fileName;		/* The name of the image file. */
-    struct Tcl_Obj *format;	/* User-specified format string, or NULL. */
+    Tcl_Obj *format;		/* User-specified format string, or NULL. */
     int *widthPtr, *heightPtr;	/* The dimensions of the image are
 				 * returned here if the file is a valid
 				 * JPEG file. */
@@ -651,8 +651,8 @@ ChnMatchJPEG(chan, fileName, format, widthPtr, heightPtr)
 
 static int
 ObjMatchJPEG(dataObj, format, widthPtr, heightPtr)
-    struct Tcl_Obj *dataObj;	/* the object containing the image data */
-    struct Tcl_Obj *format;	/* User-specified format object, or NULL. */
+    Tcl_Obj *dataObj;		/* the object containing the image data */
+    Tcl_Obj *format;		/* User-specified format object, or NULL. */
     int *widthPtr, *heightPtr;	/* The dimensions of the image are
 				 * returned here if the string is a valid
 				 * JPEG image. */
@@ -766,7 +766,7 @@ ChnReadJPEG(interp, chan, fileName, format, imageHandle, destX, destY,
     Tcl_Interp *interp;		/* Interpreter to use for reporting errors. */
     Tcl_Channel chan;		/* The image channel, open for reading. */
     char *fileName;		/* The name of the image file. */
-    struct Tcl_Obj *format;	/* User-specified format string, or NULL. */
+    Tcl_Obj *format;		/* User-specified format string, or NULL. */
     Tk_PhotoHandle imageHandle;	/* The photo image to write into. */
     int destX, destY;		/* Coordinates of top-left pixel in
 				 * photo image to be written to. */
@@ -836,8 +836,8 @@ static int
 ObjReadJPEG(interp, data, format, imageHandle, destX, destY,
 	width, height, srcX, srcY)
     Tcl_Interp *interp;		/* Interpreter to use for reporting errors. */
-    struct Tcl_Obj *data;	/* Object containing the image data. */
-    struct Tcl_Obj *format;	/* User-specified format object, or NULL. */
+    Tcl_Obj *data;		/* Object containing the image data. */
+    Tcl_Obj *format;		/* User-specified format object, or NULL. */
     Tk_PhotoHandle imageHandle;	/* The photo image to write into. */
     int destX, destY;		/* Coordinates of top-left pixel in
 				 * photo image to be written to. */
@@ -908,7 +908,7 @@ CommonReadJPEG(interp, cinfo, format, imageHandle, destX, destY,
 	width, height, srcX, srcY)
     Tcl_Interp *interp;		/* Interpreter to use for reporting errors. */
     j_decompress_ptr cinfo;	/* Already-constructed decompress struct. */
-    struct Tcl_Obj *format;	/* User-specified format string, or NULL. */
+    Tcl_Obj *format;		/* User-specified format string, or NULL. */
     Tk_PhotoHandle imageHandle;	/* The photo image to write into. */
     int destX, destY;		/* Coordinates of top-left pixel in
 				 * photo image to be written to. */
@@ -922,8 +922,8 @@ CommonReadJPEG(interp, cinfo, format, imageHandle, destX, destY,
     myblock bl;
 #define block bl.ck
     JSAMPARRAY buffer;		/* Output row buffer */
-    int argc, i, index;
-    char **argv = NULL;
+    int objc, i, index;
+    Tcl_Obj **objv = (Tcl_Obj **) NULL;
 
     /* Ready to read header data. */
     jpeg.read_header(cinfo, TRUE);
@@ -936,14 +936,13 @@ CommonReadJPEG(interp, cinfo, format, imageHandle, destX, destY,
     }
 
     /* Process format parameters. */
-    if (ImgListObjGetElements(interp, format, &argc, &argv) != TCL_OK) {
+    if (ImgListObjGetElements(interp, format, &objc, &objv) != TCL_OK) {
 	return TCL_ERROR;
     }
-    if (argc) {
-	for (i=1; i<argc; i++) {
-	    if (ImgGetIndexFromObj(interp, argv[i], jpegReadOptions, "format option",
+    if (objc) {
+	for (i=1; i<objc; i++) {
+	    if (Tcl_GetIndexFromObj(interp, objv[i], jpegReadOptions, "format option",
 		    0, &index)!=TCL_OK) {
-		ckfree((char *) argv);
 		return TCL_ERROR;
 	    }
 	    switch (index) {
@@ -962,7 +961,6 @@ CommonReadJPEG(interp, cinfo, format, imageHandle, destX, destY,
 		}
 	    }
 	}
-	ckfree((char *) argv);
     }
 
     jpeg.start_decompress(cinfo);
@@ -1060,7 +1058,7 @@ static int
 ChnWriteJPEG(interp, fileName, format, blockPtr)
     Tcl_Interp *interp;
     char *fileName;
-    struct Tcl_Obj *format;
+    Tcl_Obj *format;
     Tk_PhotoImageBlock *blockPtr;
 {
     struct jpeg_compress_struct cinfo; /* libjpeg's parameter structure */
@@ -1072,13 +1070,10 @@ ChnWriteJPEG(interp, fileName, format, blockPtr)
 	return TCL_ERROR;
     }
 
-    chan = Tcl_OpenFileChannel(interp, fileName, "w", 0644);
+    chan = ImgOpenFileChannel(interp, fileName, 0644);
     if (!chan) {
 	return TCL_ERROR;
     }
-    if (Tcl_SetChannelOption(interp, chan, "-translation", "binary") != TCL_OK) {
-	return TCL_ERROR;
-    }
 
     /* Initialize JPEG error handler */
     /* We set up the normal JPEG error routines, then override error_exit. */
@@ -1134,7 +1129,7 @@ static int
 StringWriteJPEG(interp, dataPtr, format, blockPtr)
     Tcl_Interp *interp;
     Tcl_DString *dataPtr;
-    struct Tcl_Obj *format;
+    Tcl_Obj *format;
     Tk_PhotoImageBlock *blockPtr;
 {
     struct jpeg_compress_struct cinfo; /* libjpeg's parameter structure */
@@ -1191,7 +1186,7 @@ static int
 CommonWriteJPEG(interp, cinfo, format, blockPtr)
     Tcl_Interp *interp;
     j_compress_ptr cinfo;	
-    struct Tcl_Obj *format;
+    Tcl_Obj *format;
     Tk_PhotoImageBlock *blockPtr;
 {
     static char *jpegWriteOptions[] = {"-grayscale", "-optimize",
@@ -1202,8 +1197,8 @@ CommonWriteJPEG(interp, cinfo, format, blockPtr)
     int w, h;
     int greenOffset, blueOffset, alphaOffset;
     unsigned char *pixelPtr, *pixLinePtr;
-    int argc, i, index, grayscale = 0;
-    char **argv = NULL;
+    int objc, i, index, grayscale = 0;
+    Tcl_Obj **objv = (Tcl_Obj **) NULL;
 
     greenOffset = blockPtr->offset[1] - blockPtr->offset[0];
     blueOffset = blockPtr->offset[2] - blockPtr->offset[0];
@@ -1227,13 +1222,13 @@ CommonWriteJPEG(interp, cinfo, format, blockPtr)
 
     /* Parse options, if any, and alter default parameters */
 
-    if (ImgListObjGetElements(interp, format, &argc, &argv) != TCL_OK)
+    if (ImgListObjGetElements(interp, format, &objc, &objv) != TCL_OK) {
 	return TCL_ERROR;
-    if (argc) {
-      for (i=1; i<argc; i++) {
-	    if (ImgGetIndexFromObj(interp, argv[i], jpegWriteOptions,
+    }
+    if (objc) {
+	for (i=1; i<objc; i++) {
+	    if (Tcl_GetIndexFromObj(interp, objv[i], jpegWriteOptions,
 		    "format option", 0, &index)!=TCL_OK) {
-		ckfree((char *) argv);
 		return TCL_ERROR;
 	    }
 	    switch (index) {
@@ -1254,14 +1249,12 @@ CommonWriteJPEG(interp, cinfo, format, blockPtr)
 		}
 		case 3: {
 		    int quality = 0;
-		    if (++i >= argc) {
+		    if (++i >= objc) {
 			Tcl_AppendResult(interp, "No value for option \"",
-				argv[--i], "\"", (char *) NULL);
-			ckfree((char *) argv);
+				Tcl_GetStringFromObj(objv[--i], (int *) NULL), "\"", (char *) NULL);
 			return TCL_ERROR;
 		    }
-		    if (Tcl_GetInt(interp, argv[i], &quality) != TCL_OK) {
-			ckfree((char *) argv);
+		    if (Tcl_GetIntFromObj(interp, objv[i], &quality) != TCL_OK) {
 			return TCL_ERROR;
 		    }
 		    jpeg.set_quality(cinfo, quality, FALSE);
@@ -1269,22 +1262,19 @@ CommonWriteJPEG(interp, cinfo, format, blockPtr)
 		}
 		case 4: {
 		    int smooth = 0;
-		    if (++i >= argc) {
+		    if (++i >= objc) {
 			Tcl_AppendResult(interp, "No value for option \"",
-				argv[--i], "\"", (char *) NULL);
-			ckfree((char *) argv);
+				Tcl_GetStringFromObj(objv[--i], (int *) NULL), "\"", (char *) NULL);
 			return TCL_ERROR;
 		    }
-		    if (Tcl_GetInt(interp, argv[i], &smooth) != TCL_OK) {
-			ckfree((char *) argv);
+		    if (Tcl_GetIntFromObj(interp, objv[i], &smooth) != TCL_OK) {
 			return TCL_ERROR;
 		    }
 		    cinfo->smoothing_factor = smooth;
 		    break;
 		}
 	    }
-      }
-      ckfree((char *) argv);
+	}
     }
 
     pixLinePtr = blockPtr->pixelPtr + blockPtr->offset[0];
@@ -1347,7 +1337,7 @@ CommonWriteJPEG(interp, cinfo, format, blockPtr)
 static void
 jpeg_obj_src (cinfo, dataObj)
     j_decompress_ptr cinfo;
-    struct Tcl_Obj *dataObj;
+    Tcl_Obj *dataObj;
 {
   src_ptr src;
 
