@@ -46,19 +46,19 @@
 
 
 static int ChnMatchPNG _ANSI_ARGS_((Tcl_Channel chan, char *fileName,
-	struct Tcl_Obj *format, int *widthPtr, int *heightPtr));
-static int ObjMatchPNG _ANSI_ARGS_((struct Tcl_Obj *dataObj,
-	struct Tcl_Obj *format, int *widthPtr, int *heightPtr));
+	Tcl_Obj *format, int *widthPtr, int *heightPtr));
+static int ObjMatchPNG _ANSI_ARGS_((Tcl_Obj *dataObj,
+	Tcl_Obj *format, int *widthPtr, int *heightPtr));
 static int ChnReadPNG _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Channel chan,
-	char *fileName, struct Tcl_Obj *format, Tk_PhotoHandle imageHandle,
+	char *fileName, Tcl_Obj *format, Tk_PhotoHandle imageHandle,
 	int destX, int destY, int width, int height, int srcX, int srcY));
-static int ObjReadPNG _ANSI_ARGS_((Tcl_Interp *interp, struct Tcl_Obj *dataObj,
-	struct Tcl_Obj *format, Tk_PhotoHandle imageHandle,
+static int ObjReadPNG _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj *dataObj,
+	Tcl_Obj *format, Tk_PhotoHandle imageHandle,
 	int destX, int destY, int width, int height, int srcX, int srcY));
 static int ChnWritePNG _ANSI_ARGS_((Tcl_Interp *interp, char *filename,
-	struct Tcl_Obj *format, Tk_PhotoImageBlock *blockPtr));
+	Tcl_Obj *format, Tk_PhotoImageBlock *blockPtr));
 static int StringWritePNG _ANSI_ARGS_((Tcl_Interp *interp,
-	Tcl_DString *dataPtr, struct Tcl_Obj *format,
+	Tcl_DString *dataPtr, Tcl_Obj *format,
 	Tk_PhotoImageBlock *blockPtr));
 
 Tk_PhotoImageFormat imgFmtPNG = {
@@ -77,11 +77,11 @@ Tk_PhotoImageFormat imgFmtPNG = {
 
 static int CommonMatchPNG _ANSI_ARGS_((MFile *handle, int *widthPtr,
 	int *heightPtr));
-static int CommonReadPNG _ANSI_ARGS_((png_structp png_ptr, struct Tcl_Obj *format,
+static int CommonReadPNG _ANSI_ARGS_((png_structp png_ptr, Tcl_Obj *format,
 	Tk_PhotoHandle imageHandle, int destX, int destY, int width,
 	int height, int srcX, int srcY));
 static int CommonWritePNG _ANSI_ARGS_((Tcl_Interp *interp, png_structp png_ptr,
-	png_infop info_ptr, struct Tcl_Obj *format,
+	png_infop info_ptr, Tcl_Obj *format,
 	Tk_PhotoImageBlock *blockPtr));
 static void tk_png_error _ANSI_ARGS_((png_structp, png_const_charp));
 static void tk_png_warning _ANSI_ARGS_((png_structp, png_const_charp));
@@ -131,6 +131,12 @@ static struct PngFunctions {
     void (* set_expand) _ANSI_ARGS_((png_structp));
     void (* set_filler) _ANSI_ARGS_((png_structp, png_uint_32, int));
     void (* set_strip_16) _ANSI_ARGS_((png_structp));
+    png_uint_32 (* get_sRGB) _ANSI_ARGS_((png_structp, png_infop, int *));
+    void (* set_sRGB) _ANSI_ARGS_((png_structp, png_infop, int));
+    png_uint_32 (* get_gAMA) _ANSI_ARGS_((png_structp, png_infop, double *));
+    void (* set_gAMA) PNGARG((png_structp png_ptr, png_infop, double));
+    void (* set_gamma) _ANSI_ARGS_((png_structp, double, double));
+    void (* set_sRGB_gAMA_and_cHRM) _ANSI_ARGS_((png_structp, png_infop, int));
 } png = {0};
 
 static char *symbols[] = {
@@ -163,6 +169,12 @@ static char *symbols[] = {
     "png_set_expand",
     "png_set_filler",
     "png_set_strip_16",
+    "png_get_sRGB",
+    "png_set_sRGB",
+    "png_get_gAMA",
+    "png_set_gAMA",
+    "png_set_gamma",
+    "png_set_sRGB_gAMA_and_cHRM",
     (char *) NULL
 };
 
@@ -222,7 +234,7 @@ tk_png_write(png_ptr, data, length)
 static int ChnMatchPNG(chan, fileName, format, widthPtr, heightPtr)
     Tcl_Channel chan;
     char *fileName;
-    struct Tcl_Obj *format;
+    Tcl_Obj *format;
     int *widthPtr, *heightPtr;
 {
     MFile handle;
@@ -234,8 +246,8 @@ static int ChnMatchPNG(chan, fileName, format, widthPtr, heightPtr)
 }
 
 static int ObjMatchPNG(dataObj, format, widthPtr, heightPtr)
-    struct Tcl_Obj *dataObj;
-    struct Tcl_Obj *format;
+    Tcl_Obj *dataObj;
+    Tcl_Obj *format;
     int *widthPtr, *heightPtr;
 {
     MFile handle;
@@ -280,7 +292,7 @@ static int ChnReadPNG(interp, chan, fileName, format, imageHandle,
     Tcl_Interp *interp;
     Tcl_Channel chan;
     char *fileName;
-    struct Tcl_Obj *format;
+    Tcl_Obj *format;
     Tk_PhotoHandle imageHandle;
     int destX, destY;
     int width, height;
@@ -313,8 +325,8 @@ static int ChnReadPNG(interp, chan, fileName, format, imageHandle,
 static int ObjReadPNG(interp, dataObj, format, imageHandle,
 	destX, destY, width, height, srcX, srcY)
     Tcl_Interp *interp;
-    struct Tcl_Obj *dataObj;
-    struct Tcl_Obj *format;
+    Tcl_Obj *dataObj;
+    Tcl_Obj *format;
     Tk_PhotoHandle imageHandle;
     int destX, destY;
     int width, height;
@@ -354,7 +366,7 @@ typedef struct myblock {
 static int CommonReadPNG(png_ptr, format, imageHandle, destX, destY,
 	width, height, srcX, srcY)
     png_structp png_ptr;
-    struct Tcl_Obj *format;
+    Tcl_Obj *format;
     Tk_PhotoHandle imageHandle;
     int destX, destY;
     int width, height;
@@ -367,6 +379,7 @@ static int CommonReadPNG(png_ptr, format, imageHandle, destX, destY,
     unsigned int I;
     png_uint_32 info_width, info_height;
     int bit_depth, color_type, interlace_type;
+    int intent;
 
     info_ptr=png.create_info_struct(png_ptr);
     if (!info_ptr) {
@@ -385,7 +398,6 @@ static int CommonReadPNG(png_ptr, format, imageHandle, destX, destY,
 	return TCL_ERROR;
     }
 
-
     png.read_info(png_ptr,info_ptr);
 
     png.get_IHDR(png_ptr, info_ptr, &info_width, &info_height, &bit_depth,
@@ -439,6 +451,16 @@ static int CommonReadPNG(png_ptr, format, imageHandle, destX, destY,
 	block.offset[3] = 0;
     }
 
+    if (png.get_sRGB && png.get_sRGB(png_ptr, info_ptr, &intent)) {
+	png.set_sRGB(png_ptr, info_ptr, intent);
+    } else if (png.get_gAMA) {
+	double gamma;
+	if (!png.get_gAMA(png_ptr, info_ptr, &gamma)) {
+	    gamma = 0.45455;
+	}
+	png.set_gamma(png_ptr, 1.0, gamma);
+    }
+
     png_data= (char **) ckalloc(sizeof(char *) * info_height +
 	    info_height * block.pitch);
 
@@ -465,7 +487,7 @@ static int CommonReadPNG(png_ptr, format, imageHandle, destX, destY,
 static int ChnWritePNG(interp, filename, format, blockPtr)
     Tcl_Interp *interp;
     char *filename;
-    struct Tcl_Obj *format;
+    Tcl_Obj *format;
     Tk_PhotoImageBlock *blockPtr;
 {
     FILE *outfile = NULL;
@@ -517,7 +539,7 @@ static int ChnWritePNG(interp, filename, format, blockPtr)
 static int StringWritePNG(interp, dataPtr, format, blockPtr)
     Tcl_Interp *interp;
     Tcl_DString *dataPtr;
-    struct Tcl_Obj *format;
+    Tcl_Obj *format;
     Tk_PhotoImageBlock *blockPtr;
 {
     png_structp png_ptr;
@@ -556,16 +578,16 @@ static int CommonWritePNG(interp, png_ptr, info_ptr, format, blockPtr)
     Tcl_Interp *interp;
     png_structp png_ptr;
     png_infop info_ptr;
-    struct Tcl_Obj *format;
+    Tcl_Obj *format;
     Tk_PhotoImageBlock *blockPtr;
 {
     int greenOffset, blueOffset, alphaOffset;
     int tagcount = 0;
-    char **tags = NULL;
+    Tcl_Obj **tags = (Tcl_Obj **) NULL;
     int I, pass, number_passes, color_type;  
     int newPixelSize;
     png_bytep row_pointers;
-    png_textp text = NULL;
+    png_textp text = (png_textp) NULL;
 
     if (ImgListObjGetElements(interp, format, &tagcount, &tags) != TCL_OK) {
 	return TCL_ERROR;
@@ -573,9 +595,6 @@ static int CommonWritePNG(interp, png_ptr, info_ptr, format, blockPtr)
     tagcount = (tagcount > 1) ? (tagcount/2 - 1) : 0;
 
     if (setjmp(*(jmp_buf *)png_ptr)) {
-	if (tags) {
-	    ckfree((char *) tags);
-	}
 	if (text) {
 	    ckfree((char *) text);
 	}
@@ -623,13 +642,18 @@ static int CommonWritePNG(interp, png_ptr, info_ptr, format, blockPtr)
 	    color_type, PNG_INTERLACE_ADAM7, PNG_COMPRESSION_TYPE_BASE,
 	    PNG_FILTER_TYPE_BASE);
 
+    if (png.set_gAMA) {
+	png.set_gAMA(png_ptr, info_ptr, 1.0);
+    }
+
     if (tagcount > 0) {
 	png_text text;
 	for(I=0;I<tagcount;I++) {
+	    int length;
 	    text.compression = 0;
-	    text.key = tags[2*I+1];
-	    text.text = tags[2*I+2];
-	    text.text_length=strlen(tags[2*I+2]);
+	    text.key = Tcl_GetStringFromObj(tags[2*I+1], (int *) NULL);
+	    text.text = Tcl_GetStringFromObj(tags[2*I+2], &length);
+	    text.text_length = length;
 	    if (text.text_length>COMPRESS_THRESHOLD) { 
 		text.compression = -1;
 	    }
@@ -647,7 +671,7 @@ static int CommonWritePNG(interp, png_ptr, info_ptr, format, blockPtr)
 	row_pointers = (png_bytep)
 		ckalloc(blockPtr->width * newPixelSize);
 	for (pass = 0; pass < number_passes; pass++) {
-	    for(I=0;I<blockPtr->height;I++) {
+	    for(I=0; I<blockPtr->height; I++) {
 		src = (png_bytep) blockPtr->pixelPtr
 			+ I * blockPtr->pitch + blockPtr->offset[0];
 		dst = row_pointers;
@@ -673,9 +697,6 @@ static int CommonWritePNG(interp, png_ptr, info_ptr, format, blockPtr)
     if (text) {
 	ckfree((char *) text);
     }
-    if (tags) {
-	ckfree((char *) tags);
-    }
     png.destroy_write_struct(&png_ptr,&info_ptr);
 
     return(TCL_OK);
