@@ -13,9 +13,10 @@
 #include <imgInt.h>
 #include <ctype.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <imgPmap.h>
 
-#ifdef __WIN32__
+#if defined(__WIN32__) && !defined (__GNUC__)
 #define strncasecmp strnicmp
 #endif
 
@@ -31,14 +32,18 @@ extern int	strncmp _ANSI_ARGS_((CONST char *s1, CONST char *s2,
 extern char *	strncpy _ANSI_ARGS_((char *dst, CONST char *src,
 			    size_t numChars));
 
+#ifndef TCL_STUB_MAGIC
 EXTERN void		panic _ANSI_ARGS_(TCL_VARARGS(char *,format));
+#endif
+
+#define UCHAR(c) ((unsigned char) (c))
 
 /*
  * Prototypes for procedures used only locally in this file:
  */
 
 static int		ImgXpmCreate _ANSI_ARGS_((Tcl_Interp *interp,
-			    char *name, int argc, struct Tcl_Obj *CONST objv[],
+			    char *name, int argc, Tcl_Obj *objv[],
 			    Tk_ImageType *typePtr, Tk_ImageMaster master,
 			    ClientData *clientDataPtr));
 static ClientData	ImgXpmGet _ANSI_ARGS_((Tk_Window tkwin,
@@ -119,7 +124,7 @@ ImgXpmCreate(interp, name, argc, objv, typePtr, master, clientDataPtr)
 				 * image. */
     char *name;			/* Name to use for image. */
     int argc;			/* Number of arguments. */
-    struct Tcl_Obj *CONST objv[];/* Argument strings for options (doesn't
+    Tcl_Obj *objv[];		/* Argument strings for options (doesn't
 				 * include image name or type). */
     Tk_ImageType *typePtr;	/* Pointer to our type record (not used). */
     Tk_ImageMaster master;	/* Token for image, to be used by us in
@@ -130,16 +135,16 @@ ImgXpmCreate(interp, name, argc, objv, typePtr, master, clientDataPtr)
     PixmapMaster *masterPtr;
     int i;
     char *argvbuf[10];
-    char **argv = argvbuf;
+    char **args = argvbuf;
 
     /*
      * Convert the objc/objv arguments into string equivalent.
      */
     if (argc > 10) {
-	argv = (char **) ckalloc(argc * sizeof(char *));
+	args = (char **) ckalloc(argc * sizeof(char *));
     }
     for (i = 0; i < argc; i++) {
-	argv[i] = ImgGetStringFromObj(objv[i], NULL);
+	args[i] = ImgGetStringFromObj(objv[i], NULL);
     }
 
     masterPtr = (PixmapMaster *) ckalloc(sizeof(PixmapMaster));
@@ -154,16 +159,16 @@ ImgXpmCreate(interp, name, argc, objv, typePtr, master, clientDataPtr)
     masterPtr->isDataAlloced = 0;
     masterPtr->instancePtr = NULL;
 
-    if (ImgXpmConfigureMaster(masterPtr, argc, argv, 0) != TCL_OK) {
+    if (ImgXpmConfigureMaster(masterPtr, argc, args, 0) != TCL_OK) {
 	ImgXpmDelete((ClientData) masterPtr);
-	if (argv != argvbuf) {
-	    ckfree((char *) argv);
+	if (args != argvbuf) {
+	    ckfree((char *) args);
 	}
 	return TCL_ERROR;
     }
     *clientDataPtr = (ClientData) masterPtr;
-    if (argv != argvbuf) {
-	ckfree((char *) argv);
+    if (args != argvbuf) {
+	ckfree((char *) args);
     }
     return TCL_OK;
 }
@@ -384,7 +389,7 @@ static char ** ImgXpmGetDataFromString(interp, string, numLines_return)
      * to skip the leading blanks is good for using in-line XPM data in TCL
      * scripts
      */
-    while (isspace(*string)) {
+    while (isspace(UCHAR(*string))) {
 	++ string;
     }
 
@@ -449,7 +454,7 @@ static char ** ImgXpmGetDataFromString(interp, string, numLines_return)
 		continue;
 	    }
 
-	    if (isspace(*p)) {
+	    if (isspace(UCHAR(*p))) {
 		*p = ' ';
 	    }
 	    else if (*p == ',') {
@@ -508,13 +513,10 @@ static char ** ImgXpmGetDataFromFile(interp, fileName, numLines_return)
     char ** data = (char **) NULL;
     char *cmdBuffer = NULL;
 
-    chan = Tcl_OpenFileChannel(interp, fileName, "r", 0);
+    chan = ImgOpenFileChannel(interp, fileName, 0);
     if (!chan) {
 	return (char **) NULL;
     }
-    if (Tcl_SetChannelOption(interp, chan, "-translation", "binary") != TCL_OK) {
-	return (char **) NULL;
-    }
 
     size = Tcl_Seek(chan, 0, SEEK_END);
     if (size > 0) {
@@ -549,34 +551,34 @@ GetType(colorDefn, type_ret)
     char * p = colorDefn;
 
     /* skip white spaces */
-    while (*p && isspace(*p)) {
+    while (*p && isspace(UCHAR(*p))) {
 	p ++;
     }
 
     /* parse the type */
     if (p[0] != '\0' && p[0] == 'm' &&
-	p[1] != '\0' && isspace(p[1])) {
+	p[1] != '\0' && isspace(UCHAR(p[1]))) {
 	*type_ret = XPM_MONO;
 	p += 2;
     }
     else if (p[0] != '\0' && p[0] == 'g' &&
 	     p[1] != '\0' && p[1] == '4' &&
-	     p[2] != '\0' && isspace(p[2])) {
+	     p[2] != '\0' && isspace(UCHAR(p[2]))) {
 	*type_ret = XPM_GRAY_4;
 	p += 3;
     }
     else if (p[0] != '\0' && p[0] == 'g' &&
-	     p[1] != '\0' && isspace(p[1])) {
+	     p[1] != '\0' && isspace(UCHAR(p[1]))) {
 	*type_ret = XPM_GRAY;
 	p += 2;
     }
     else if (p[0] != '\0' && p[0] == 'c' &&
-	     p[1] != '\0' && isspace(p[1])) {
+	     p[1] != '\0' && isspace(UCHAR(p[1]))) {
 	*type_ret = XPM_COLOR;
 	p += 2;
     }
     else if (p[0] != '\0' && p[0] == 's' &&
-	     p[1] != '\0' && isspace(p[1])) {
+	     p[1] != '\0' && isspace(UCHAR(p[1]))) {
 	*type_ret = XPM_SYMBOLIC;
 	p += 2;
     }
@@ -613,7 +615,7 @@ GetColor(colorDefn, colorName, type_ret)
     }
 
     /* skip white spaces */
-    while (*colorDefn && isspace(*colorDefn)) {
+    while (*colorDefn && isspace(UCHAR(*colorDefn))) {
 	colorDefn ++;
     }
 
@@ -622,7 +624,7 @@ GetColor(colorDefn, colorName, type_ret)
     while (1) {
 	int dummy;
 
-	while (*colorDefn && !isspace(*colorDefn)) {
+	while (*colorDefn && !isspace(UCHAR(*colorDefn))) {
 	    *p++ = *colorDefn++;
 	}
 
@@ -634,7 +636,7 @@ GetColor(colorDefn, colorName, type_ret)
 	    /* the next string should also be considered as a part of a color
 	     * name */
 	    
-	    while (*colorDefn && isspace(*colorDefn)) {
+	    while (*colorDefn && isspace(UCHAR(*colorDefn))) {
 		*p++ = *colorDefn++;
 	    }
 	} else {
