@@ -8,7 +8,7 @@
  *	CMG (Computer Management Group) Arnhem B.V.
  *	email: Jan.Nijtmans@wxs.nl (private)
  *	       Jan.Nijtmans@cmg.nl (work)
- *	url:   http:/home.wxs.nl/~nijtmans/
+ *	url:   http://home.wxs.nl/~nijtmans/
  *
  * (with some code stolen from the XPM image type and the GIF handler)
  *
@@ -25,10 +25,11 @@
  */
 
 #include "imgInt.h"
+#include <X11/Xlib.h>
 #include <ctype.h>
 #include <stdlib.h>
 
-#ifdef __WIN32__
+#if defined(__WIN32__) && !defined(__GNUC__)
 #define strncasecmp strnicmp
 #else
 extern VOID *	memcpy _ANSI_ARGS_((VOID *s1, CONST VOID *s2,
@@ -66,31 +67,31 @@ extern char *	strstr _ANSI_ARGS_((CONST char *string,
  */
 
 static int		ChanMatchXPM _ANSI_ARGS_((Tcl_Channel chan,
-			    char *fileName, struct Tcl_Obj *format, int *widthPtr,
+			    char *fileName, Tcl_Obj *format, int *widthPtr,
 			    int *heightPtr));
-static int      	ObjMatchXPM _ANSI_ARGS_((struct Tcl_Obj *dataObj,
-		            struct Tcl_Obj *format, int *widthPtr, int *heightPtr));
+static int      	ObjMatchXPM _ANSI_ARGS_((Tcl_Obj *dataObj,
+		            Tcl_Obj *format, int *widthPtr, int *heightPtr));
 static int		ChanReadXPM  _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tcl_Channel chan, char *fileName,
-			    struct Tcl_Obj *format, Tk_PhotoHandle imageHandle,
+			    Tcl_Obj *format, Tk_PhotoHandle imageHandle,
 			    int destX, int destY, int width, int height,
 			    int srcX, int srcY));
 static int	        ObjReadXPM _ANSI_ARGS_((Tcl_Interp *interp,
-			    struct Tcl_Obj *dataObj, struct Tcl_Obj *format,
+			    Tcl_Obj *dataObj, Tcl_Obj *format,
               		    Tk_PhotoHandle imageHandle, int destX, int destY,
 		            int width, int height, int srcX, int srcY));
 static int		CommonReadXPM _ANSI_ARGS_((Tcl_Interp *interp, MFile *handle,
-			    struct Tcl_Obj *format, Tk_PhotoHandle imageHandle,
+			    Tcl_Obj *format, Tk_PhotoHandle imageHandle,
 			    int destX, int destY, int width, int height,
 			    int srcX, int srcY));
 static int	        StringWriteXPM _ANSI_ARGS_((Tcl_Interp *interp,
-               		    Tcl_DString *dataPtr, struct Tcl_Obj *format,
+               		    Tcl_DString *dataPtr, Tcl_Obj *format,
 		            Tk_PhotoImageBlock *blockPtr));
 static int              FileWriteXPM _ANSI_ARGS_((Tcl_Interp *interp,
-                            char *fileName, struct Tcl_Obj *format,
+                            char *fileName, Tcl_Obj *format,
                             Tk_PhotoImageBlock *blockPtr));
 static int		CommonWriteXPM _ANSI_ARGS_((Tcl_Interp *interp, char *fileName,
-			    Tcl_DString *dataPtr, struct Tcl_Obj *format,
+			    Tcl_DString *dataPtr, Tcl_Obj *format,
 			    Tk_PhotoImageBlock *blockPtr));
 
 Tk_PhotoImageFormat imgFmtXPM = {
@@ -174,8 +175,8 @@ Gets(handle, buffer, size)
  */
 static int
 ObjMatchXPM(dataObj, format, widthPtr, heightPtr)
-    struct Tcl_Obj *dataObj;	/* The data supplied by the image */
-    struct Tcl_Obj *format;	/* User-specified format object, or NULL. */
+    Tcl_Obj *dataObj;		/* The data supplied by the image */
+    Tcl_Obj *format;		/* User-specified format object, or NULL. */
     int *widthPtr, *heightPtr;	/* The dimensions of the image are
 				 * returned here if the file is a valid
 				 * raw XPM file. */
@@ -212,7 +213,7 @@ static int
 ChanMatchXPM(chan, fileName, format, widthPtr, heightPtr)
     Tcl_Channel chan;		/* The image channel, open for reading. */
     char *fileName;		/* The name of the image file. */
-    struct Tcl_Obj *format;	/* User-specified format object, or NULL. */
+    Tcl_Obj *format;		/* User-specified format object, or NULL. */
     int *widthPtr, *heightPtr;	/* The dimensions of the image are
 				 * returned here if the file is a valid
 				 * raw XPM file. */
@@ -258,7 +259,7 @@ CommonReadXPM(interp, handle, format, imageHandle, destX, destY,
 	width, height, srcX, srcY)
     Tcl_Interp *interp;		/* Interpreter to use for reporting errors. */
     MFile *handle;		/* The image channel, open for reading. */
-    struct Tcl_Obj *format;	/* User-specified format object, or NULL. */
+    Tcl_Obj *format;		/* User-specified format object, or NULL. */
     Tk_PhotoHandle imageHandle;	/* The photo image to write into. */
     int destX, destY;		/* Coordinates of top-left pixel in
 				 * photo image to be written to. */
@@ -536,7 +537,7 @@ ChanReadXPM(interp, chan, fileName, format, imageHandle, destX, destY,
     Tcl_Interp *interp;		/* Interpreter to use for reporting errors. */
     Tcl_Channel chan;		/* The image channel, open for reading. */
     char *fileName;		/* The name of the image file. */
-    struct Tcl_Obj *format;	/* User-specified format object, or NULL. */
+    Tcl_Obj *format;		/* User-specified format object, or NULL. */
     Tk_PhotoHandle imageHandle;	/* The photo image to write into. */
     int destX, destY;		/* Coordinates of top-left pixel in
 				 * photo image to be written to. */
@@ -578,8 +579,8 @@ static int
 ObjReadXPM(interp, data, format, imageHandle, destX, destY,
 	width, height, srcX, srcY)
     Tcl_Interp *interp;		/* Interpreter to use for reporting errors. */
-    struct Tcl_Obj *data;
-    struct Tcl_Obj *format;	/* User-specified format object, or NULL. */
+    Tcl_Obj *data;
+    Tcl_Obj *format;		/* User-specified format object, or NULL. */
     Tk_PhotoHandle imageHandle;	/* The photo image to write into. */
     int destX, destY;		/* Coordinates of top-left pixel in
 				 * photo image to be written to. */
@@ -696,34 +697,34 @@ static char * GetType(colorDefn, type_ret)
     char * p = colorDefn;
 
     /* skip white spaces */
-    while (*p && isspace(*p)) {
+    while (*p && isspace(UCHAR(*p))) {
 	p ++;
     }
 
     /* parse the type */
     if (p[0] != '\0' && p[0] == 'm' &&
-	p[1] != '\0' && isspace(p[1])) {
+	p[1] != '\0' && isspace(UCHAR(p[1]))) {
 	*type_ret = XPM_MONO;
 	p += 2;
     }
     else if (p[0] != '\0' && p[0] == 'g' &&
 	     p[1] != '\0' && p[1] == '4' &&
-	     p[2] != '\0' && isspace(p[2])) {
+	     p[2] != '\0' && isspace(UCHAR(p[2]))) {
 	*type_ret = XPM_GRAY_4;
 	p += 3;
     }
     else if (p[0] != '\0' && p[0] == 'g' &&
-	     p[1] != '\0' && isspace(p[1])) {
+	     p[1] != '\0' && isspace(UCHAR(p[1]))) {
 	*type_ret = XPM_GRAY;
 	p += 2;
     }
     else if (p[0] != '\0' && p[0] == 'c' &&
-	     p[1] != '\0' && isspace(p[1])) {
+	     p[1] != '\0' && isspace(UCHAR(p[1]))) {
 	*type_ret = XPM_COLOR;
 	p += 2;
     }
     else if (p[0] != '\0' && p[0] == 's' &&
-	     p[1] != '\0' && isspace(p[1])) {
+	     p[1] != '\0' && isspace(UCHAR(p[1]))) {
 	*type_ret = XPM_SYMBOLIC;
 	p += 2;
     }
@@ -757,7 +758,7 @@ static char * GetColor(colorDefn, colorName, type_ret)
     }
 
     /* skip white spaces */
-    while (*colorDefn && isspace(*colorDefn)) {
+    while (*colorDefn && isspace(UCHAR(*colorDefn))) {
 	colorDefn ++;
     }
 
@@ -766,7 +767,7 @@ static char * GetColor(colorDefn, colorName, type_ret)
     while (1) {
 	int dummy;
 
-	while (*colorDefn && !isspace(*colorDefn)) {
+	while (*colorDefn && !isspace(UCHAR(*colorDefn))) {
 	    *p++ = *colorDefn++;
 	}
 
@@ -778,7 +779,7 @@ static char * GetColor(colorDefn, colorName, type_ret)
 	    /* the next string should also be considered as a part of a color
 	     * name */
 	    
-	    while (*colorDefn && isspace(*colorDefn)) {
+	    while (*colorDefn && isspace(UCHAR(*colorDefn))) {
 		*p++ = *colorDefn++;
 	    }
 	} else {
@@ -816,7 +817,7 @@ static int
 FileWriteXPM(interp, fileName, format, blockPtr)
     Tcl_Interp *interp;
     char *fileName;
-    struct Tcl_Obj *format;
+    Tcl_Obj *format;
     Tk_PhotoImageBlock *blockPtr;
 {
     return CommonWriteXPM(interp, fileName, (Tcl_DString *)NULL, format, blockPtr);
@@ -843,7 +844,7 @@ static int
 StringWriteXPM(interp, dataPtr, format, blockPtr) 
     Tcl_Interp *interp;
     Tcl_DString *dataPtr;
-    struct Tcl_Obj *format;
+    Tcl_Obj *format;
     Tk_PhotoImageBlock *blockPtr;
 {
   return CommonWriteXPM(interp, "unknown", dataPtr, format, blockPtr);
@@ -880,7 +881,7 @@ CommonWriteXPM(interp, fileName, dataPtr, format, blockPtr)
     Tcl_Interp *interp;
     char *fileName;
     Tcl_DString *dataPtr;
-    struct Tcl_Obj *format;    
+    Tcl_Obj *format;    
     Tk_PhotoImageBlock *blockPtr;
 {
     int x, y, i;
